<?php

class PageModules extends Page
{
  function __construct(System $System)
  {
    parent::__construct($System);
    $this->FullTitle = 'Správa modulů';
    $this->ShortTitle = 'Moduly';
    $this->ParentClass = 'PagePortal';
  }

  function ShowList(): string
  {
    $Output = '';
    $DbResult = $this->Database->query('SELECT COUNT(*) FROM `Module`');
    $DbRow = $DbResult->fetch_row();
    $PageList = GetPageList('Modules', $DbRow[0]);

    $Output .= $PageList['Output'];
    $Output .= '<table class="WideTable" style="font-size: small;">';

    $TableColumns = array(
      array('Name' => 'Name', 'Title' => 'Jméno'),
      array('Name' => 'Creator', 'Title' => 'Tvůrce'),
      array('Name' => 'Version', 'Title' => 'Verze'),
      array('Name' => 'License', 'Title' => 'Licence'),
      array('Name' => 'Installed', 'Title' => 'Instalováno'),
      array('Name' => 'Description', 'Title' => 'Popis'),
      array('Name' => 'Dependencies', 'Title' => 'Závislosti'),
      array('Name' => '', 'Title' => 'Akce'),
    );
    $Order = GetOrderTableHeader('Modules', $TableColumns, 'Name', 0);
    $Output .= $Order['Output'];
    $Query = 'SELECT *, (SELECT GROUP_CONCAT(`T1`.`Name` SEPARATOR ", ") FROM `ModuleLink` '.
        'LEFT JOIN `Module` AS `T1` ON `T1`.`Id` = `ModuleLink`.`LinkedModule` '.
        'WHERE `ModuleLink`.`Module` = `Module`.`Id`) AS `Dependencies` '.
        'FROM `Module` '.$Order['SQL'].$PageList['SQLLimit'];

    $DbResult = $this->Database->query($Query);
    while ($Module = $DbResult->fetch_assoc())
    {
      if ($Module['Dependencies'] != '') $Dependencies = $Module['Dependencies'];
      else $Dependencies = '&nbsp;';
      if ($Module['Installed'] == 1) $Installed = 'Ano';
      else $Installed = 'Ne';
      if ($Module['Installed'] == 1) $Actions = '<a href="?A=Uninstall&amp;Id='.$Module['Id'].'">Odinstalovat</a>';
      else $Actions = '<a href="?A=Install&amp;Id='.$Module['Id'].'">Instalovat</a>';
      $Output .= '<tr><td>'.$Module['Name'].'</td>'.
          '<td>'.$Module['Creator'].'</td>'.
          '<td>'.$Module['Version'].'</td>'.
          '<td>'.$Module['License'].'</td>'.
          '<td>'.$Installed.'</td>'.
          '<td>'.$Module['Description'].'</td>'.
          '<td>'.$Dependencies.'</td>'.
          '<td>'.$Actions.'</td></tr>';
    }
    $Output .= '</table>';
    $Output .= $PageList['Output'];
    $Output .= '<p><a href="?A=SaveToDb">Uložit do databáze</a></p>';
    return $Output;
  }

  function Show(): string
  {
    $Output = '';
    if (array_key_exists('A', $_GET))
    {
      if ($_GET['A'] == 'SaveToDb')
      {
        $Output .= ModuleSystem::Cast($this->System->GetModule('System'))->SaveToDatabase();
        $Output .= $this->SystemMessage('Načtení modulů', 'Seznam modulů v databázi zaktualizován');
      } else
      if ($_GET['A'] == 'Install')
      {
        $this->System->ModuleManager->LoadModules(false);
        $ModuleName = $this->System->ModuleManager->SearchModuleById($_GET['Id']);
        if ($ModuleName != '')
        {
          $this->System->ModuleManager->GetModule($ModuleName)->Install();
        } else $Output .= 'Modul id '.$_GET['Id'].' nenalezen';

      } else
      if ($_GET['A'] == 'Uninstall')
      {
        $ModuleName = $this->System->ModuleManager->SearchModuleById($_GET['Id']);
        if ($ModuleName != '')
        {
          $this->System->ModuleManager->GetModule($ModuleName)->UnInstall();
        } else $Output .= 'Modul id '.$_GET['Id'].' nenalezen';
      } else $Output .= 'Neplatná akce';
    }
    $Output .= $this->ShowList();
    return $Output;
  }
}

class ModuleSystem extends AppModule
{
  public bool $InstalledChecked;

  function __construct(System $System)
  {
    parent::__construct($System);
    $this->Name = 'System';
    $this->Version = '1.0';
    $this->Creator = 'Chronos';
    $this->License = 'GNU/GPL';
    $this->Description = 'Base system module';
    $this->Dependencies = array();
  }

  function DoInstall(): void
  {
    $this->Database->query('CREATE TABLE IF NOT EXISTS `SystemVersion` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `Version` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Description` datetime NOT NULL,
  PRIMARY KEY (`Id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_ci AUTO_INCREMENT=1;');
    $this->Database->query('CREATE TABLE IF NOT EXISTS `SystemModule` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `Name` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Creator` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Version` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `License` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Installed` int(11) NOT NULL,
  `Description` text COLLATE utf8_czech_ci NOT NULL,
  PRIMARY KEY (`Id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_ci AUTO_INCREMENT=1;');

    $this->Database->query('CREATE TABLE IF NOT EXISTS `SystemModuleDependency` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `Module` int(11) NOT NULL,
  `DependencyModule` int(11) NOT NULL,
  PRIMARY KEY (`Id`),
  KEY (`Module`),
  KEY (`DependencyModule`),
  UNIQUE (`Module` , `DependencyModule`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_ci AUTO_INCREMENT=1;');
    $this->Database->query('ALTER TABLE `SystemModuleDependency` ADD CONSTRAINT `SystemModuleDependency_ibfk_1` FOREIGN KEY ( `Module` ) REFERENCES `SystemModule` (`Id`)');
    $this->Database->query('ALTER TABLE `SystemModuleDependency` ADD CONSTRAINT `SystemModuleDependency_ibfk_2` FOREIGN KEY ( `DependencyModule` ) REFERENCES `SystemModule` (`Id`)');

    $this->Database->query('CREATE TABLE IF NOT EXISTS `SystemModel` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `Name` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Module` int(11) NOT NULL,
  KEY (`Module`),
  PRIMARY KEY (`Id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_ci AUTO_INCREMENT=1;');
    $this->Database->query('ALTER TABLE `SystemModel` ADD CONSTRAINT `SystemModel_ibfk_1` FOREIGN KEY ( `Module` ) REFERENCES `SystemModule` (`Id`)');

    $this->Database->query('CREATE TABLE IF NOT EXISTS `SystemModelProperty` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `Name` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Type` varchar(255) COLLATE utf8_czech_ci NOT NULL,
  `Model` int(11) NOT NULL,
  KEY (`Model`),
  PRIMARY KEY (`Id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_ci AUTO_INCREMENT=1;');
    $this->Database->query('ALTER TABLE `SystemModelProperty` ADD CONSTRAINT `SystemModelProperty_ibfk_1` FOREIGN KEY ( `Model` ) REFERENCES `SystemModel` (`Id`)');
  }

  function DoUnInstall(): void
  {
    // Delete tables with reverse order
    $this->Database->query('ALTER TABLE `SystemModelProperty` DROP FOREIGN KEY `SystemModelProperty_ibfk_1`');
    $this->Database->query('DROP TABLE IF EXISTS `SystemModelProperty`');
    $this->Database->query('ALTER TABLE `SystemModel` DROP FOREIGN KEY `SystemModel_ibfk_1`');
    $this->Database->query('DROP TABLE IF EXISTS `SystemModel`');
    $this->Database->query('ALTER TABLE `SystemModuleDependency` DROP FOREIGN KEY `SystemModuleDependency_ibfk_1`');
    $this->Database->query('ALTER TABLE `SystemModuleDependency` DROP FOREIGN KEY `SystemModuleDependency_ibfk_2`');
    $this->Database->query('DROP TABLE IF EXISTS `SystemModuleDependency`');
    $this->Database->query('DROP TABLE IF EXISTS `SystemModule`');
    $this->Database->query('DROP TABLE IF EXISTS `SystemVersion`');
  }

  function DoStart(): void
  {
    $this->System->RegisterPage(['module'], 'PageModules');
    $this->System->FormManager->RegisterClass('Action', array(
      'Title' => 'Akce',
      'Table' => 'Action',
      'Items' => array(
        //'Name' => array('Type' => 'String', 'Caption' => 'Název', 'Default' => ''),
        'Title' => array('Type' => 'String', 'Caption' => 'Název', 'Default' => ''),
        'URL' => array('Type' => 'Hyperlink', 'Caption' => 'Odkaz', 'Default' => ''),
        'Icon' => array('Type' => 'TActionIcon', 'Caption' => 'Ikony', 'Default' => '', 'Null' => true),
        'Type' => array('Type' => 'TActionType', 'Caption' => 'Typ', 'Default' => ''),
        'Group' => array('Type' => 'TActionGroup', 'Caption' => 'Skupina', 'Default' => '', 'Null' => true),
        'PermissionOperation' => array('Type' => 'TPermissionOperation', 'Caption' => 'Operace oprávnění', 'Default' => ''),
        'Enable' => array('Type' => 'Boolean', 'Caption' => 'Povolení', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterClass('ActionIcon', array(
      'Title' => 'Ikony akcí',
      'Table' => 'ActionIcon',
      'Items' => array(
        'Name' => array('Type' => 'Image', 'Caption' => 'Název souboru', 'Default' => ''),
        'Items' => array('Type' => 'TActionListIcon', 'Caption' => 'Položky', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterClass('ActionGroup', array(
      'Title' => 'Skupiny akcí',
      'Table' => 'ActionGroup',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Název', 'Default' => ''),
        'Items' => array('Type' => 'TActionListGroup', 'Caption' => 'Položky', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterClass('ActionType', array(
      'Title' => 'Typy akcí',
      'Table' => 'ActionType',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Název', 'Default' => ''),
        'Items' => array('Type' => 'TActionListType', 'Caption' => 'Položky', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterClass('Module', array(
      'Title' => 'Moduly',
      'Table' => 'Module',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Systémové jméno', 'Default' => ''),
        'Title' => array('Type' => 'String', 'Caption' => 'Název', 'Default' => ''),
        'Description' => array('Type' => 'Text', 'Caption' => 'Popis', 'Default' => ''),
        'Version' => array('Type' => 'String', 'Caption' => 'Verze', 'Default' => ''),
        'License' => array('Type' => 'String', 'Caption' => 'Licence', 'Default' => ''),
        'Creator' => array('Type' => 'String', 'Caption' => 'Tvůrce', 'Default' => ''),
        'HomePage' => array('Type' => 'Hyperlink', 'Caption' => 'Domovské stránky', 'Default' => ''),
        'Installed' => array('Type' => 'Boolean', 'Caption' => 'Instalováno', 'Default' => '', 'ReadOnly' => true),
        'Models' => array('Type' => 'TModelListModule', 'Caption' => 'Modely', 'Default' => ''),
        'Links' => array('Type' => 'TModuleLinkListModule', 'Caption' => 'Vazby', 'Default' => ''),
      ),
      'Actions' => array(
        array('Caption' => 'Aktualizovat z disku', 'URL' => '/module/?A=SaveToDb'),
      ),
    ));
    $this->System->FormManager->RegisterFormType('TModule', array(
      'Type' => 'Reference',
      'Table' => 'Module',
      'Id' => 'Id',
      'Name' => 'Title',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TModelListModule', array(
      'Type' => 'ManyToOne',
      'Table' => 'Model',
      'Id' => 'Id',
      'Ref' => 'Module',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterClass('Model', array(
      'Title' => 'Modely',
      'Table' => 'Model',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Systémové jméno', 'Default' => ''),
        'Title' => array('Type' => 'String', 'Caption' => 'Název', 'Default' => ''),
        'Module' => array('Type' => 'TModule', 'Caption' => 'Module', 'Default' => ''),
        'Query' => array('Type' => 'String', 'Caption' => 'SQL dotaz', 'Default' => ''),
        'DefaultSortColumn' => array('Type' => 'String', 'Caption' => 'Výchozí sloupce řazení', 'Default' => ''),
        'DefaultSortOrder' => array('Type' => 'Text', 'Caption' => 'Výchozí směr řazení', 'Default' => ''),
        'Fields' => array('Type' => 'TModelFieldListModel', 'Caption' => 'Pole', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterFormType('TModel', array(
      'Type' => 'Reference',
      'Table' => 'Model',
      'Id' => 'Id',
      'Name' => 'Title',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TModelFieldListModel', array(
      'Type' => 'ManyToOne',
      'Table' => 'ModelField',
      'Id' => 'Id',
      'Ref' => 'Model',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterClass('ModelField', array(
      'Title' => 'Pole modelu',
      'Table' => 'ModelField',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Systémové jméno', 'Default' => ''),
        'Title' => array('Type' => 'String', 'Caption' => 'Název', 'Default' => ''),
        'Model' => array('Type' => 'TModel', 'Caption' => 'Model', 'Default' => ''),
        'Query' => array('Type' => 'String', 'Caption' => 'SQL dotaz', 'Default' => ''),
        'Type' => array('Type' => 'String', 'Caption' => 'Typ', 'Default' => ''),
        'DefaultValue' => array('Type' => 'String', 'Caption' => 'Výchozí hodnota', 'Default' => ''),
        'IsNull' => array('Type' => 'Boolean', 'Caption' => 'Také nulová hodnota', 'Default' => ''),
        'Suffix' => array('Type' => 'String', 'Caption' => 'Text za', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterFormType('TModuleLink', array(
      'Type' => 'Reference',
      'Table' => 'ModuleLink',
      'Id' => 'Id',
      'Name' => 'Module',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TModuleLinkListModule', array(
      'Type' => 'ManyToOne',
      'Table' => 'ModuleLink',
      'Id' => 'Id',
      'Ref' => 'Module',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterClass('ModuleLink', array(
      'Title' => 'Vazby modulu',
      'Table' => 'ModuleLink',
      'Items' => array(
        'Module' => array('Type' => 'TModule', 'Caption' => 'Modul', 'Default' => ''),
        'LinkedModule' => array('Type' => 'TModule', 'Caption' => 'Vázaný modul', 'Default' => ''),
        'Type' => array('Type' => 'String', 'Caption' => 'Typ vazby', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterClass('Language', array(
      'Title' => 'Jazyky',
      'Table' => 'Language',
      'DefaultSortColumn' => 'Name',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Jméno', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterClass('UnitOfMeasure', array(
      'Title' => 'Měrné jednotky',
      'Table' => 'UnitOfMeasure',
      'DefaultSortColumn' => 'Name',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Jméno', 'Default' => ''),
        'Unit' => array('Type' => 'String', 'Caption' => 'Jednotka', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterClass('Country', array(
      'Title' => 'Země',
      'Table' => 'Country',
      'DefaultSortColumn' => 'Name',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Jméno', 'Default' => ''),
      ),
    ));
    $this->System->FormManager->RegisterFormType('TCountry', array(
      'Type' => 'Reference',
      'Table' => 'Country',
      'Id' => 'Id',
      'Name' => 'Name',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TUnitOfMeasure', array(
      'Type' => 'Reference',
      'Table' => 'UnitOfMeasure',
      'Id' => 'Id',
      'Name' => 'Name',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TLanguage', array(
      'Type' => 'Reference',
      'Table' => 'Language',
      'Id' => 'Id',
      'Name' => 'Name',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TAction', array(
      'Type' => 'Reference',
      'Table' => 'Action',
      'Id' => 'Id',
      'Name' => 'Title',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TActionIcon', array(
      'Type' => 'Reference',
      'Table' => 'ActionIcon',
      'Id' => 'Id',
      'Name' => 'Name',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TActionType', array(
      'Type' => 'Reference',
      'Table' => 'ActionType',
      'Id' => 'Id',
      'Name' => 'Name',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TActionGroup', array(
      'Type' => 'Reference',
      'Table' => 'ActionGroup',
      'Id' => 'Id',
      'Name' => 'Name',
      'Filter' => '1',
    ));
    $this->System->FormManager->RegisterFormType('TModule', array(
      'Type' => 'Reference',
      'Table' => 'Module',
      'Id' => 'Id',
      'Name' => 'Name',
      'Filter' => '1',
    ));

    //$this->Manager->OnModuleChange = array($this, 'ModuleChange');
    //$this->LoadFromDatabase();
  }

  function IsInstalled(): bool
  {
    if ($this->InstalledChecked == false)
    {
      $DbResult = $this->Database->query('SELECT table_name FROM information_schema.tables
WHERE table_schema = "'.$this->Database->Database.'" AND table_name = "SystemVersion";');
      if ($DbResult->num_rows > 0) $this->Installed = true;
        else $this->Installed = false;
      $this->InstalledChecked = true;
    }
    return $this->Installed;
  }

  function ModuleChange($Module): void
  {
    //if ($this->IsInstalled())
    {

      if ($Module->IsInstalled()) $Installed = 1;
        else $Installed = 0;
      $this->Database->query('UPDATE `Module` SET `Installed`=1 WHERE `Name`="'.$Module->Name.'"');
    }
  }

  function LoadFromDatabase(): void
  {
    //DebugLog('Loading modules...');
    $this->Modules = array();
    $Query = 'SELECT `Id`, `Name`,`Installed` FROM `Module`';
    $DbResult = $this->Database->query($Query);
    while ($Module = $DbResult->fetch_array())
    {
      //echo($Module['Name'].',');
      include_once('Modules/'.$Module['Name'].'/'.$Module['Name'].'.php');
      $ModuleClassName = 'Module'.$Module['Name'];
      $NewModule = new $ModuleClassName($this->Database, $this->Manager);
      $NewModule->Id = $Module['Id'];
      $NewModule->Installed = $Module['Installed'];
      $this->Manager->RegisterModule($NewModule);
    }
  }

  function SaveToDatabase(): string
  {
    $Output = '';
    $Modules = array();
    $DbResult = $this->Database->query('SELECT * FROM `Module`');
    while ($DbRow = $DbResult->fetch_assoc())
    {
      $Modules[$DbRow['Name']] = $DbRow;
      if ($this->System->ModuleManager->ModulePresent($DbRow['Name']))
        $this->System->ModuleManager->GetModule($DbRow['Name'])->Id = $DbRow['Id'];
    }

    // Add missing
    foreach ($this->System->ModuleManager->Modules as $Module)
    {
      if (!array_key_exists($Module->Name, $Modules))
      {
        $this->Database->insert('Module', array('Name' => $Module->Name,
          'Version' => $Module->Version, 'Creator' => $Module->Creator,
          'HomePage' => $Module->HomePage, 'Title' => $Module->Title,
          'Description' => $Module->Description, 'License' => $Module->License,
          'Installed' => $Module->Installed));
        $this->System->ModuleManager->GetModule($Module->Name)->Id = $this->Database->insert_id;
      }
      else $this->Database->update('Module', 'Name = "'.$Module->Name.'"', array(
        'Version' => $Module->Version, 'Creator' => $Module->Creator,
        'HomePage' => $Module->HomePage, 'Title' => $Module->Title,
        'Description' => $Module->Description, 'License' => $Module->License,
        'Installed' => $Module->Installed));
    }

    // Remove exceeding
    foreach ($Modules as $Module)
    if (!$this->System->ModuleManager->ModulePresent($Module['Name']))
    {
      $Output .= 'Removing module '.$Module['Name'].' from list</br/>';
      $this->Database->query('DELETE FROM `ModuleLink` WHERE `Module` = '.$Module['Id']);
      $this->Database->query('DELETE FROM `ModuleLink` WHERE `LinkedModule` = '.$Module['Id']);
      $DbResult = $this->Database->query('SELECT Id FROM `PermissionOperation` WHERE `Module` = '.$Module['Id']);
      while ($DbRow = $DbResult->fetch_assoc())
      {
        $this->Database->query('DELETE FROM `PermissionGroupAssignment` WHERE `AssignedOperation` = '.$DbRow['Id']);
        $this->Database->query('DELETE FROM `PermissionUserAssignment` WHERE `AssignedOperation` = '.$DbRow['Id']);
      }
      $this->Database->query('DELETE FROM `PermissionOperation` WHERE `Module` = '.$Module['Id']);
      $this->Database->query('DELETE FROM `Model` WHERE `Module` = '.$Module['Id']);
      $DbResult = $this->Database->query('SELECT Id FROM `Model` WHERE `Module` = '.$Module['Id']);
      while ($DbRow = $DbResult->fetch_assoc())
        $this->Database->query('DELETE FROM `ModelField` WHERE `Model` = '.$DbRow['Id']);
      $this->Database->query('DELETE FROM `Module` WHERE `Id` = '.$Module['Id']);
    }

    // Reload dependencies
    $DbDependency = array();
    $DbResult = $this->Database->query('SELECT * FROM `ModuleLink`');
    while ($DbRow = $DbResult->fetch_assoc())
      $DbDependency[$DbRow['Module']][] = $DbRow['LinkedModule'];

    foreach ($this->System->ModuleManager->Modules as $Module)
    {
      // Add missing
      foreach ($Module->Dependencies as $Dependency)
      {
        if (!array_key_exists($Module->Id, $DbDependency) or
        !in_array($this->System->ModuleManager->GetModule($Dependency)->Id, $DbDependency[$Module->Id]))
        {
          if ($this->System->ModuleManager->ModulePresent($Dependency))
            $DependencyId = $this->System->ModuleManager->GetModule($Dependency)->Id;
            else throw new Exception('Dependent module '.$Dependency.' not found');
          $this->Database->insert('ModuleLink', array('Module' => $Module->Id,
            'LinkedModule' => $DependencyId, 'Type' => 'DependOn'));
        }
      }

      // Remove exceeding
      if (array_key_exists($Module->Id, $DbDependency))
      foreach ($DbDependency[$Module->Id] as $Dep)
      {
        $DepModName = $this->System->ModuleManager->SearchModuleById($Dep);
        if (!in_array($DepModName, $Module->Dependencies))
        $this->Database->query('DELETE FROM `ModuleLink` WHERE `Module` = '.
          $Module->Id.' AND LinkedModule='.$Dep);
      }
    }
    return $Output;
  }

  static function Cast(AppModule $AppModule): ModuleSystem
  {
    if ($AppModule instanceof ModuleSystem)
    {
      return $AppModule;
    }
    throw new Exception('Expected ModuleSystem type but got '.gettype($AppModule));
  }
}
